/* 
   Unix SMB/CIFS implementation.

   POSIX NTVFS backend - open and close

   Copyright (C) Andrew Tridgell 2004

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "includes.h"
#include "vfs_posix.h"
#include "ioctl.h"

/*
  old ioctl interface 
*/
static NTSTATUS pvfs_ioctl_old(struct ntvfs_module_context *ntvfs,
			struct smbsrv_request *req, union smb_ioctl *io)
{
	return NT_STATUS_UNSUCCESSFUL;
}

/*
  nt ioctl interface 
*/
static NTSTATUS pvfs_ntioctl(struct ntvfs_module_context *ntvfs,
			     struct smbsrv_request *req, union smb_ioctl *io)
{
	struct pvfs_state *pvfs = ntvfs->private_data;
	struct pvfs_file *f;

	f = pvfs_find_fd(pvfs, req, io->ntioctl.in.fnum);
	if (!f) {
		return NT_STATUS_INVALID_HANDLE;
	}

	switch (io->ntioctl.in.function) {
	case FSCTL_SET_SPARSE:
		/* maybe some posix systems have a way of marking
		   a file non-sparse? */
		io->ntioctl.out.blob = data_blob(NULL, 0);
		return NT_STATUS_OK;
	}

	return NT_STATUS_NOT_SUPPORTED;
}

/*
  ioctl interface 
*/
NTSTATUS pvfs_ioctl(struct ntvfs_module_context *ntvfs,
		    struct smbsrv_request *req, union smb_ioctl *io)
{
	NTSTATUS status = NT_STATUS_UNSUCCESSFUL;

	switch (io->generic.level) {
	case RAW_IOCTL_IOCTL:
		status = pvfs_ioctl_old(ntvfs, req, io);
		break;

	case RAW_IOCTL_NTIOCTL:
		status = pvfs_ntioctl(ntvfs, req, io);
		break;
	}

	return status;
}

